/*
 *             Twidere - Twitter client for Android
 *
 *  Copyright (C) 2012-2017 Mariotaku Lee <mariotaku.lee@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.ktextension

import java.lang.ref.WeakReference
import kotlin.properties.ReadOnlyProperty
import kotlin.reflect.KProperty

class WeakDelegate<T>(obj: T?) {

    private var weakRef: WeakReference<T>? = null

    init {
        weakRef = if (obj != null) WeakReference(obj) else null
    }

    operator fun getValue(thisRef: Any?, property: KProperty<*>): T? = weakRef?.get()

    operator fun setValue(thisRef: Any?, property: KProperty<*>, value: T?) {
        weakRef = if (value != null) WeakReference(value) else null
    }
}

private class ReadOnlyWeakDelegate<T>(private val ref: WeakReference<T>) : ReadOnlyProperty<Any, T?> {

    override fun getValue(thisRef: Any, property: KProperty<*>): T? = ref.get()
}


fun <T> weak(obj: T? = null): WeakDelegate<T> = WeakDelegate(obj)

operator fun <T> WeakReference<T>.provideDelegate(thisRef: Any, prop: KProperty<*>): ReadOnlyProperty<Any, T?> =
        ReadOnlyWeakDelegate(this)