/*
 *             Twidere - Twitter client for Android
 *
 *  Copyright (C) 2012-2017 Mariotaku Lee <mariotaku.lee@gmail.com>
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.data.fetcher

import org.mariotaku.microblog.library.*
import org.mariotaku.microblog.library.model.Paging
import org.mariotaku.microblog.library.model.mastodon.LinkHeaderList
import org.mariotaku.microblog.library.model.microblog.Status
import org.mariotaku.twidere.alias.MastodonStatus
import org.mariotaku.twidere.exception.RequiredFieldNotFoundException
import org.mariotaku.twidere.model.AccountDetails
import org.mariotaku.twidere.model.UserKey
import org.mariotaku.twidere.model.timeline.TimelineFilter

class UserFavoritesFetcher(val userKey: UserKey?, val userScreenName: String?) : StatusesFetcher {

    override fun forTwitter(account: AccountDetails, twitter: Twitter, paging: Paging, filter: TimelineFilter?): List<Status> {
        return getMicroBlogUserFavorites(twitter, paging)
    }

    override fun forStatusNet(account: AccountDetails, statusNet: StatusNet, paging: Paging, filter: TimelineFilter?): List<Status> {
        return getMicroBlogUserFavorites(statusNet, paging)
    }

    override fun forFanfou(account: AccountDetails, fanfou: Fanfou, paging: Paging, filter: TimelineFilter?): List<Status> {
        return getMicroBlogUserFavorites(fanfou, paging)
    }

    override fun forMastodon(account: AccountDetails, mastodon: Mastodon, paging: Paging, filter: TimelineFilter?): LinkHeaderList<MastodonStatus> {
        if (userKey != account.key) {
            return LinkHeaderList()
        }
        return mastodon.getFavourites(paging)
    }

    private fun getMicroBlogUserFavorites(microBlog: MicroBlog, paging: Paging): List<Status> {
        return when {
            userKey != null -> microBlog.getFavorites(userKey.id, paging)
            userScreenName != null -> microBlog.getFavoritesByScreenName(userScreenName, paging)
            else -> throw RequiredFieldNotFoundException("user_id", "screen_name")
        }
    }
}