/*
 *             Twidere - Twitter client for Android
 *
 *  Copyright (C) 2012-2017 Mariotaku Lee <mariotaku.lee@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.util.preference

import android.content.SharedPreferences
import org.mariotaku.commons.collection.MultiMap
import org.mariotaku.twidere.singleton.PreferencesSingleton
import org.mariotaku.twidere.util.lang.ApplicationContextSingletonHolder

class PreferenceChangeNotifier private constructor(preferences: SharedPreferences) {

    private val listenersMap = MultiMap<String, (String) -> Unit>()
    private val changeListener = SharedPreferences.OnSharedPreferenceChangeListener { _, key ->
        listenersMap[key]?.forEach { it.invoke(key) }
    }

    init {
        preferences.registerOnSharedPreferenceChangeListener(changeListener)
    }

    fun register(key: String, callback: (String) -> Unit) {
        listenersMap.add(key, callback)
    }

    fun register(vararg keys: String, callback: (String) -> Unit) {
        keys.forEach { key -> listenersMap.add(key, callback) }
    }

    companion object : ApplicationContextSingletonHolder<PreferenceChangeNotifier>({
        PreferenceChangeNotifier(PreferencesSingleton.get(it))
    })
}
